@@ -44,7 +44,7 @@ class OverlayLabel(QtWidgets.QLabel):
 
 
 class Console(QtWidgets.QPlainTextEdit):
-    def __init__(self, prompt='>> ', startup_message='', parent=None):
+    def __init__(self, prompt='>>> ', parent=None):
         QtWidgets.QPlainTextEdit.__init__(self, parent)
 
         self.prompt = prompt
@@ -56,7 +56,6 @@ class Console(QtWidgets.QPlainTextEdit):
         self.setWordWrapMode(QtGui.QTextOption.WrapAnywhere)
         self.setUndoRedoEnabled(False)
         self.document().setDefaultFont(QtGui.QFont(MONOSPACE_FONT, 10, QtGui.QFont.Normal))
-        self.showMessage(startup_message)
 
         self.updateNamespace({'run':self.run_script})
         self.set_json(False)
@@ -91,17 +90,18 @@ class Console(QtWidgets.QPlainTextEdit):
 
     def showMessage(self, message):
         self.appendPlainText(message)
-        self.newPrompt()
+        self.newPrompt('')
 
     def clear(self):
+        curr_line = self.getCommand()
         self.setPlainText('')
-        self.newPrompt()
+        self.newPrompt(curr_line)
 
-    def newPrompt(self):
+    def newPrompt(self, curr_line):
         if self.construct:
-            prompt = '.' * len(self.prompt)
+            prompt = '... '
         else:
-            prompt = self.prompt
+            prompt = self.prompt + curr_line
 
         self.completions_pos = self.textCursor().position()
         self.completions_visible = False
@@ -160,9 +160,8 @@ class Console(QtWidgets.QPlainTextEdit):
 
     def getConstruct(self, command):
         if self.construct:
-            prev_command = self.construct[-1]
             self.construct.append(command)
-            if not prev_command and not command:
+            if not command:
                 ret_val = '\n'.join(self.construct)
                 self.construct = []
                 return ret_val
@@ -175,12 +174,6 @@ class Console(QtWidgets.QPlainTextEdit):
             else:
                 return command
 
-    def getHistory(self):
-        return self.history
-
-    def setHisory(self, history):
-        self.history = history
-
     def addToHistory(self, command):
         if command[0:1] == ' ':
             return
@@ -212,11 +205,6 @@ class Console(QtWidgets.QPlainTextEdit):
         for i in range(len(self.prompt) + position):
             self.moveCursor(QtGui.QTextCursor.Right)
 
-    def register_command(self, c, func):
-        methods = { c: func}
-        self.updateNamespace(methods)
-
-
     def runCommand(self):
         command = self.getCommand()
         self.addToHistory(command)
@@ -244,7 +232,7 @@ class Console(QtWidgets.QPlainTextEdit):
             if type(self.namespace.get(command)) == type(lambda:None):
                 self.appendPlainText("'{}' is a function. Type '{}()' to use it in the Python console."
                                      .format(command, command))
-                self.newPrompt()
+                self.newPrompt('')
                 return
 
             sys.stdout = stdoutProxy(self.appendPlainText)
@@ -269,7 +257,7 @@ class Console(QtWidgets.QPlainTextEdit):
                     traceback_lines.pop(i)
                 self.appendPlainText('\n'.join(traceback_lines))
             sys.stdout = tmp_stdout
-        self.newPrompt()
+        self.newPrompt('')
         self.set_json(False)
 
 
@@ -346,17 +334,3 @@ class Console(QtWidgets.QPlainTextEdit):
                 self.setCommand(beginning + p)
             else:
                 self.show_completions(completions)
-
-
-welcome_message = '''
-   ---------------------------------------------------------------
-     Welcome to a primitive Python interpreter.
-   ---------------------------------------------------------------
-'''
-
-if __name__ == '__main__':
-    app = QtWidgets.QApplication(sys.argv)
-    console = Console(startup_message=welcome_message)
-    console.updateNamespace({'myVar1' : app, 'myVar2' : 1234})
-    console.show()
-    sys.exit(app.exec_())
