@@ -7,28 +7,30 @@ import queue
 import traceback
 import os
 import webbrowser
-
+from decimal import Decimal
 from functools import partial, lru_cache
-from typing import NamedTuple, Callable, Optional, TYPE_CHECKING, Union, List, Dict
+from typing import (NamedTuple, Callable, Optional, TYPE_CHECKING, Union, List, Dict, Any,
+                    Sequence, Iterable)
 
 from PyQt5.QtGui import (QFont, QColor, QCursor, QPixmap, QStandardItem,
-                         QPalette, QIcon, QFontMetrics)
+                         QPalette, QIcon, QFontMetrics, QShowEvent)
 from PyQt5.QtCore import (Qt, QPersistentModelIndex, QModelIndex, pyqtSignal,
                           QCoreApplication, QItemSelectionModel, QThread,
-                          QSortFilterProxyModel, QSize, QLocale)
+                          QSortFilterProxyModel, QSize, QLocale, QAbstractItemModel)
 from PyQt5.QtWidgets import (QPushButton, QLabel, QMessageBox, QHBoxLayout,
                              QAbstractItemView, QVBoxLayout, QLineEdit,
                              QStyle, QDialog, QGroupBox, QButtonGroup, QRadioButton,
                              QFileDialog, QWidget, QToolButton, QTreeView, QPlainTextEdit,
-                             QHeaderView, QApplication, QToolTip, QTreeWidget, QStyledItemDelegate)
+                             QHeaderView, QApplication, QToolTip, QTreeWidget, QStyledItemDelegate,
+                             QMenu)
 
 from electrum.i18n import _, languages
-from electrum.util import (FileImportFailed, FileExportFailed,
-                           resource_path)
-from electrum.paymentrequest import PR_UNPAID, PR_PAID, PR_EXPIRED
+from electrum.util import FileImportFailed, FileExportFailed, make_aiohttp_session, resource_path
+from electrum.invoices import PR_UNPAID, PR_PAID, PR_EXPIRED, PR_INFLIGHT, PR_UNKNOWN, PR_FAILED, PR_ROUTING
 
 if TYPE_CHECKING:
     from .main_window import ElectrumWindow
+    from .installwizard import InstallWizard
 
 
 if platform.system() == 'Windows':
@@ -42,23 +44,23 @@ else:
 dialogs = []
 
 pr_icons = {
+    PR_UNKNOWN:"warning.png",
     PR_UNPAID:"unpaid.png",
     PR_PAID:"confirmed.png",
-    PR_EXPIRED:"expired.png"
+    PR_EXPIRED:"expired.png",
+    PR_INFLIGHT:"unconfirmed.png",
+    PR_FAILED:"warning.png",
+    PR_ROUTING:"unconfirmed.png",
 }
 
-pr_tooltips = {
-    PR_UNPAID:_('Pending'),
-    PR_PAID:_('Paid'),
-    PR_EXPIRED:_('Expired')
-}
 
-expiration_values = [
-    (_('1 hour'), 60*60),
-    (_('1 day'), 24*60*60),
-    (_('1 week'), 7*24*60*60),
-    (_('Never'), None)
-]
+# filter tx files in QFileDialog:
+TRANSACTION_FILE_EXTENSION_FILTER_ANY = "Transaction (*.txn *.psbt);;All files (*)"
+TRANSACTION_FILE_EXTENSION_FILTER_ONLY_PARTIAL_TX = "Partial Transaction (*.psbt)"
+TRANSACTION_FILE_EXTENSION_FILTER_ONLY_COMPLETE_TX = "Complete Transaction (*.txn)"
+TRANSACTION_FILE_EXTENSION_FILTER_SEPARATE = (f"{TRANSACTION_FILE_EXTENSION_FILTER_ONLY_PARTIAL_TX};;"
+                                              f"{TRANSACTION_FILE_EXTENSION_FILTER_ONLY_COMPLETE_TX};;"
+                                              f"All files (*)")
 
 
 class EnterButton(QPushButton):
@@ -68,7 +70,7 @@ class EnterButton(QPushButton):
         self.clicked.connect(func)
 
     def keyPressEvent(self, e):
-        if e.key() == Qt.Key_Return:
+        if e.key() in [ Qt.Key_Return, Qt.Key_Enter ]:
             self.func()
 
 
@@ -161,6 +163,8 @@ class Buttons(QHBoxLayout):
         QHBoxLayout.__init__(self)
         self.addStretch(1)
         for b in buttons:
+            if b is None:
+                continue
             self.addWidget(b)
 
 class CloseButton(QPushButton):
@@ -287,13 +291,14 @@ class WindowModalDialog(QDialog, MessageBoxMixin):
 class WaitingDialog(WindowModalDialog):
     '''Shows a please wait dialog whilst running a task.  It is not
     necessary to maintain a reference to this dialog.'''
-    def __init__(self, parent, message, task, on_success=None, on_error=None):
+    def __init__(self, parent: QWidget, message: str, task, on_success=None, on_error=None):
         assert parent
         if isinstance(parent, MessageBoxMixin):
             parent = parent.top_level_window()
         WindowModalDialog.__init__(self, parent, _("Please wait"))
+        self.message_label = QLabel(message)
         vbox = QVBoxLayout(self)
-        vbox.addWidget(QLabel(message))
+        vbox.addWidget(self.message_label)
         self.accepted.connect(self.on_accepted)
         self.show()
         self.thread = TaskThread(self)
@@ -306,6 +311,30 @@ class WaitingDialog(WindowModalDialog):
     def on_accepted(self):
         self.thread.stop()
 
+    def update(self, msg):
+        print(msg)
+        self.message_label.setText(msg)
+
+
+class BlockingWaitingDialog(WindowModalDialog):
+    """Shows a waiting dialog whilst running a task.
+    Should be called from the GUI thread. The GUI thread will be blocked while
+    the task is running; the point of the dialog is to provide feedback
+    to the user regarding what is going on.
+    """
+    def __init__(self, parent: QWidget, message: str, task: Callable[[], Any]):
+        assert parent
+        if isinstance(parent, MessageBoxMixin):
+            parent = parent.top_level_window()
+        WindowModalDialog.__init__(self, parent, _("Please wait"))
+        self.message_label = QLabel(message)
+        vbox = QVBoxLayout(self)
+        vbox.addWidget(self.message_label)
+        self.show()
+        QCoreApplication.processEvents()
+        task()
+        self.accept()
+
 
 def line_dialog(parent, title, label, ok_label, default=None):
     dialog = WindowModalDialog(parent, title)
@@ -439,8 +468,9 @@ def filename_field(parent, config, defaultname, select_msg):
 
     return vbox, filename_e, b1
 
+
 class ElectrumItemDelegate(QStyledItemDelegate):
-    def __init__(self, tv):
+    def __init__(self, tv: 'MyTreeView'):
         super().__init__(tv)
         self.tv = tv
         self.opened = None
@@ -450,7 +480,7 @@ class ElectrumItemDelegate(QStyledItemDelegate):
             new_text = editor.text()
             idx = QModelIndex(self.opened)
             row, col = idx.row(), idx.column()
-            _prior_text, user_role = self.tv.text_txid_from_coordinate(row, col)
+            _prior_text, user_role = self.tv.get_text_and_userrole_from_coordinate(row, col)
             # check that we didn't forget to set UserRole on an editable field
             assert user_role is not None, (row, col)
             self.tv.on_edited(idx, user_role, new_text)
@@ -461,9 +491,14 @@ class ElectrumItemDelegate(QStyledItemDelegate):
         self.opened = QPersistentModelIndex(idx)
         return super().createEditor(parent, option, idx)
 
-class MyTreeView(QTreeView):
 
-    def __init__(self, parent: 'ElectrumWindow', create_menu, stretch_column=None, editable_columns=None):
+class MyTreeView(QTreeView):
+    ROLE_CLIPBOARD_DATA = Qt.UserRole + 100
+
+    filter_columns: Iterable[int]
+
+    def __init__(self, parent: 'ElectrumWindow', create_menu, *,
+                 stretch_column=None, editable_columns=None):
         super().__init__(parent)
         self.parent = parent
         self.config = self.parent.config
@@ -473,10 +508,12 @@ class MyTreeView(QTreeView):
         self.setUniformRowHeights(True)
 
         # Control which columns are editable
-        if editable_columns is None:
+        if editable_columns is not None:
+            editable_columns = set(editable_columns)
+        elif stretch_column is not None:
             editable_columns = {stretch_column}
         else:
-            editable_columns = set(editable_columns)
+            editable_columns = {}
         self.editable_columns = editable_columns
         self.setItemDelegate(ElectrumItemDelegate(self))
         self.current_filter = ""
@@ -491,6 +528,9 @@ class MyTreeView(QTreeView):
         # only look at as many rows as currently visible.
         self.header().setResizeContentsPrecision(0)
 
+        self._pending_update = False
+        self._forced_update = False
+
     def set_editability(self, items):
         for idx, i in enumerate(items):
             i.setEditable(idx in self.editable_columns)
@@ -499,13 +539,28 @@ class MyTreeView(QTreeView):
         items = self.selectionModel().selectedIndexes()
         return list(x for x in items if x.column() == column)
 
-    def current_item_user_role(self, col) -> Optional[QStandardItem]:
+    def current_item_user_role(self, col) -> Any:
         idx = self.selectionModel().currentIndex()
         idx = idx.sibling(idx.row(), col)
-        item = self.model().itemFromIndex(idx)
+        item = self.item_from_index(idx)
         if item:
             return item.data(Qt.UserRole)
 
+    def item_from_index(self, idx: QModelIndex) -> Optional[QStandardItem]:
+        model = self.model()
+        if isinstance(model, QSortFilterProxyModel):
+            idx = model.mapToSource(idx)
+            return model.sourceModel().itemFromIndex(idx)
+        else:
+            return model.itemFromIndex(idx)
+
+    def original_model(self) -> QAbstractItemModel:
+        model = self.model()
+        if isinstance(model, QSortFilterProxyModel):
+            return model.sourceModel()
+        else:
+            return model
+
     def set_current_idx(self, set_current: QPersistentModelIndex):
         if set_current:
             assert isinstance(set_current, QPersistentModelIndex)
@@ -517,8 +572,7 @@ class MyTreeView(QTreeView):
         if not isinstance(headers, dict):  # convert to dict
             headers = dict(enumerate(headers))
         col_names = [headers[col_idx] for col_idx in sorted(headers.keys())]
-        model = self.model()
-        model.setHorizontalHeaderLabels(col_names)
+        self.original_model().setHorizontalHeaderLabels(col_names)
         self.header().setStretchLastSection(False)
         for col_idx in headers:
             sm = QHeaderView.Stretch if col_idx == self.stretch_column else QHeaderView.ResizeToContents
@@ -527,7 +581,7 @@ class MyTreeView(QTreeView):
     def keyPressEvent(self, event):
         if self.itemDelegate().opened:
             return
-        if event.key() in [ Qt.Key_F2, Qt.Key_Return ]:
+        if event.key() in [ Qt.Key_F2, Qt.Key_Return, Qt.Key_Enter ]:
             self.on_activated(self.selectionModel().currentIndex())
             return
         super().keyPressEvent(event)
@@ -558,10 +612,9 @@ class MyTreeView(QTreeView):
         """
         return False
 
-    def text_txid_from_coordinate(self, row_num, column):
-        assert not isinstance(self.model(), QSortFilterProxyModel)
+    def get_text_and_userrole_from_coordinate(self, row_num, column):
         idx = self.model().index(row_num, column)
-        item = self.model().itemFromIndex(idx)
+        item = self.item_from_index(idx)
         user_role = item.data(Qt.UserRole)
         return item.text(), user_role
 
@@ -576,7 +629,7 @@ class MyTreeView(QTreeView):
             self.setRowHidden(row_num, QModelIndex(), False)
             return
         for column in self.filter_columns:
-            txt, _ = self.text_txid_from_coordinate(row_num, column)
+            txt, _ = self.get_text_and_userrole_from_coordinate(row_num, column)
             txt = txt.lower()
             if self.current_filter in txt:
                 # the filter matched, but the date filter might apply
@@ -627,16 +680,66 @@ class MyTreeView(QTreeView):
     def toggle_toolbar(self, config=None):
         self.show_toolbar(not self.toolbar_shown, config)
 
+    def add_copy_menu(self, menu: QMenu, idx) -> QMenu:
+        cc = menu.addMenu(_("Copy"))
+        for column in self.Columns:
+            column_title = self.original_model().horizontalHeaderItem(column).text()
+            item_col = self.item_from_index(idx.sibling(idx.row(), column))
+            clipboard_data = item_col.data(self.ROLE_CLIPBOARD_DATA)
+            if clipboard_data is None:
+                clipboard_data = item_col.text().strip()
+            cc.addAction(column_title,
+                         lambda text=clipboard_data, title=column_title:
+                         self.place_text_on_clipboard(text, title=title))
+        return cc
+
+    def place_text_on_clipboard(self, text: str, *, title: str = None) -> None:
+        self.parent.do_copy(text, title=title)
+
+    def showEvent(self, e: 'QShowEvent'):
+        super().showEvent(e)
+        if e.isAccepted() and self._pending_update:
+            self._forced_update = True
+            self.update()
+            self._forced_update = False
+
+    def maybe_defer_update(self) -> bool:
+        """Returns whether we should defer an update/refresh."""
+        defer = not self.isVisible() and not self._forced_update
+        # side-effect: if we decide to defer update, the state will become stale:
+        self._pending_update = defer
+        return defer
+
+
+class MySortModel(QSortFilterProxyModel):
+    def __init__(self, parent, *, sort_role):
+        super().__init__(parent)
+        self._sort_role = sort_role
+
+    def lessThan(self, source_left: QModelIndex, source_right: QModelIndex):
+        item1 = self.sourceModel().itemFromIndex(source_left)
+        item2 = self.sourceModel().itemFromIndex(source_right)
+        data1 = item1.data(self._sort_role)
+        data2 = item2.data(self._sort_role)
+        if data1 is not None and data2 is not None:
+            return data1 < data2
+        v1 = item1.text()
+        v2 = item2.text()
+        try:
+            return Decimal(v1) < Decimal(v2)
+        except:
+            return v1 < v2
+
 
 class ButtonsWidget(QWidget):
 
     def __init__(self):
         super(QWidget, self).__init__()
-        self.buttons = []
+        self.buttons = []  # type: List[QToolButton]
 
     def resizeButtons(self):
         frameWidth = self.style().pixelMetric(QStyle.PM_DefaultFrameWidth)
-        x = self.rect().right() - frameWidth
+        x = self.rect().right() - frameWidth - 10
         y = self.rect().bottom() - frameWidth
         for button in self.buttons:
             sz = button.sizeHint()
@@ -686,6 +789,18 @@ class ButtonsTextEdit(QPlainTextEdit, ButtonsWidget):
         return o
 
 
+class PasswordLineEdit(QLineEdit):
+    def __init__(self, *args, **kwargs):
+        QLineEdit.__init__(self, *args, **kwargs)
+        self.setEchoMode(QLineEdit.Password)
+
+    def clear(self):
+        # Try to actually overwrite the memory.
+        # This is really just a best-effort thing...
+        self.setText(len(self.text()) * " ")
+        super().clear()
+
+
 class TaskThread(QThread):
     '''Thread that runs background tasks.  Callbacks are guaranteed
     to happen in the context of its parent.'''
@@ -755,8 +870,8 @@ class ColorScheme:
     YELLOW = ColorSchemeItem("#897b2a", "#ffff00")
     RED = ColorSchemeItem("#7c1111", "#f18c8c")
     BLUE = ColorSchemeItem("#123b7c", "#8cb3f2")
-    PURPLE = ColorSchemeItem("#8A2BE2", "#8A2BE2")
     DEFAULT = ColorSchemeItem("black", "white")
+    GRAY = ColorSchemeItem("gray", "gray")
 
     @staticmethod
     def has_dark_background(widget):
@@ -831,16 +946,23 @@ def export_meta_gui(electrum_window, title, exporter):
                                      .format(title, str(filename)))
 
 
-def get_parent_main_window(widget):
+def get_parent_main_window(
+        widget, *, allow_wizard: bool = False,
+) -> Union[None, 'ElectrumWindow', 'InstallWizard']:
     """Returns a reference to the ElectrumWindow this widget belongs to."""
     from .main_window import ElectrumWindow
+    from .transaction_dialog import TxDialog
+    from .installwizard import InstallWizard
     for _ in range(100):
         if widget is None:
             return None
-        if not isinstance(widget, ElectrumWindow):
-            widget = widget.parentWidget()
-        else:
+        if isinstance(widget, ElectrumWindow):
             return widget
+        if isinstance(widget, TxDialog):
+            return widget.main_window
+        if isinstance(widget, InstallWizard) and allow_wizard:
+            return widget
+        widget = widget.parentWidget()
     return None
 
 
@@ -857,22 +979,6 @@ def get_default_language():
     name = QLocale.system().name()
     return name if name in languages else 'en_UK'
 
-class FromList(QTreeWidget):
-    def __init__(self, parent, create_menu):
-        super().__init__(parent)
-        self.setHeaderHidden(True)
-        self.setMaximumHeight(300)
-        self.setContextMenuPolicy(Qt.CustomContextMenu)
-        self.customContextMenuRequested.connect(create_menu)
-        self.setUniformRowHeights(True)
-        # remove left margin
-        self.setRootIsDecorated(False)
-        self.setColumnCount(2)
-        self.header().setStretchLastSection(False)
-        sm = QHeaderView.ResizeToContents
-        self.header().setSectionResizeMode(0, sm)
-        self.header().setSectionResizeMode(1, sm)
-
 
 def char_width_in_lineedit() -> int:
     char_width = QFontMetrics(QLineEdit().font()).averageCharWidth()
