@@ -29,38 +29,50 @@ import sys
 import traceback
 import asyncio
 import socket
-from typing import Tuple, Union, List, TYPE_CHECKING, Optional
+from typing import Tuple, Union, List, TYPE_CHECKING, Optional, Set, NamedTuple, Any
 from collections import defaultdict
-from ipaddress import IPv4Network, IPv6Network, ip_address
+from ipaddress import IPv4Network, IPv6Network, ip_address, IPv6Address, IPv4Address
 import itertools
 import logging
+import hashlib
 
 import aiorpcx
-from aiorpcx import RPCSession, Notification, NetAddress
+from aiorpcx import TaskGroup
+from aiorpcx import RPCSession, Notification, NetAddress, NewlineFramer
 from aiorpcx.curio import timeout_after, TaskTimeout
 from aiorpcx.jsonrpc import JSONRPC, CodeMessageError
 from aiorpcx.rawsocket import RSClient
 import certifi
 
-from .util import ignore_exceptions, log_exceptions, bfh, SilentTaskGroup
+from .util import (ignore_exceptions, log_exceptions, bfh, SilentTaskGroup, MySocksProxy,
+                   is_integer, is_non_negative_integer, is_hash256_str, is_hex_str,
+                   is_real_number)
 from . import util
 from . import x509
 from . import pem
 from . import version
 from . import blockchain
-from .blockchain import Blockchain
+from .blockchain import Blockchain, HEADER_SIZE
 from . import constants
 from .i18n import _
 from .logging import Logger
+from .transaction import Transaction
 
 if TYPE_CHECKING:
     from .network import Network
+    from .simple_config import SimpleConfig
 
 
 ca_path = certifi.where()
 
 BUCKET_NAME_OF_ONION_SERVERS = 'onion'
 
+MAX_INCOMING_MSG_SIZE = 1_000_000  # in bytes
+
+_KNOWN_NETWORK_PROTOCOLS = {'t', 's'}
+PREFERRED_NETWORK_PROTOCOL = 's'
+assert PREFERRED_NETWORK_PROTOCOL in _KNOWN_NETWORK_PROTOCOLS
+
 
 class NetworkTimeout:
     # seconds
@@ -73,6 +85,45 @@ class NetworkTimeout:
         RELAXED = 20
         MOST_RELAXED = 60
 
+
+def assert_non_negative_integer(val: Any) -> None:
+    if not is_non_negative_integer(val):
+        raise RequestCorrupted(f'{val!r} should be a non-negative integer')
+
+
+def assert_integer(val: Any) -> None:
+    if not is_integer(val):
+        raise RequestCorrupted(f'{val!r} should be an integer')
+
+
+def assert_real_number(val: Any, *, as_str: bool = False) -> None:
+    if not is_real_number(val, as_str=as_str):
+        raise RequestCorrupted(f'{val!r} should be a number')
+
+
+def assert_hash256_str(val: Any) -> None:
+    if not is_hash256_str(val):
+        raise RequestCorrupted(f'{val!r} should be a hash256 str')
+
+
+def assert_hex_str(val: Any) -> None:
+    if not is_hex_str(val):
+        raise RequestCorrupted(f'{val!r} should be a hex str')
+
+
+def assert_dict_contains_field(d: Any, *, field_name: str) -> Any:
+    if not isinstance(d, dict):
+        raise RequestCorrupted(f'{d!r} should be a dict')
+    if field_name not in d:
+        raise RequestCorrupted(f'required field {field_name!r} missing from dict')
+    return d[field_name]
+
+
+def assert_list_or_tuple(val: Any) -> None:
+    if not isinstance(val, (list, tuple)):
+        raise RequestCorrupted(f'{val!r} should be a list or tuple')
+
+
 class NotificationSession(RPCSession):
 
     def __init__(self, *args, **kwargs):
@@ -156,8 +207,15 @@ class NotificationSession(RPCSession):
         if self.interface.debug or self.interface.network.debug:
             self.interface.logger.debug(msg)
 
+    def default_framer(self):
+        # overridden so that max_size can be customized
+        return NewlineFramer(max_size=MAX_INCOMING_MSG_SIZE)
 
-class GracefulDisconnect(Exception):
+
+class NetworkException(Exception): pass
+
+
+class GracefulDisconnect(NetworkException):
     log_level = logging.INFO
 
     def __init__(self, *args, log_level=None, **kwargs):
@@ -171,9 +229,13 @@ class RequestTimedOut(GracefulDisconnect):
         return _("Network request timed out.")
 
 
+class RequestCorrupted(Exception): pass
+
 class ErrorParsingSSLCert(Exception): pass
 class ErrorGettingSSLCertFromServer(Exception): pass
-class ConnectError(Exception): pass
+class ErrorSSLCertFingerprintMismatch(Exception): pass
+class InvalidOptionCombination(Exception): pass
+class ConnectError(NetworkException): pass
 
 
 class _RSClient(RSClient):
@@ -185,72 +247,140 @@ class _RSClient(RSClient):
             raise ConnectError(e) from e
 
 
-def deserialize_server(server_str: str) -> Tuple[str, str, str]:
-    # host might be IPv6 address, hence do rsplit:
-    host, port, protocol = str(server_str).rsplit(':', 2)
-    if not host:
-        raise ValueError('host must not be empty')
-    if protocol not in ('s', 't'):
-        raise ValueError('invalid network protocol: {}'.format(protocol))
-    int(port)  # Throw if cannot be converted to int
-    if not (0 < int(port) < 2**16):
-        raise ValueError('port {} is out of valid range'.format(port))
-    return host, port, protocol
+class ServerAddr:
+
+    def __init__(self, host: str, port: Union[int, str], *, protocol: str = None):
+        assert isinstance(host, str), repr(host)
+        if protocol is None:
+            protocol = 's'
+        if not host:
+            raise ValueError('host must not be empty')
+        if host[0] == '[' and host[-1] == ']':  # IPv6
+            host = host[1:-1]
+        try:
+            net_addr = NetAddress(host, port)  # this validates host and port
+        except Exception as e:
+            raise ValueError(f"cannot construct ServerAddr: invalid host or port (host={host}, port={port})") from e
+        if protocol not in _KNOWN_NETWORK_PROTOCOLS:
+            raise ValueError(f"invalid network protocol: {protocol}")
+        self.host = str(net_addr.host)  # canonical form (if e.g. IPv6 address)
+        self.port = int(net_addr.port)
+        self.protocol = protocol
+        self._net_addr_str = str(net_addr)
+
+    @classmethod
+    def from_str(cls, s: str) -> 'ServerAddr':
+        # host might be IPv6 address, hence do rsplit:
+        host, port, protocol = str(s).rsplit(':', 2)
+        return ServerAddr(host=host, port=port, protocol=protocol)
+
+    @classmethod
+    def from_str_with_inference(cls, s: str) -> Optional['ServerAddr']:
+        """Construct ServerAddr from str, guessing missing details.
+        Ongoing compatibility not guaranteed.
+        """
+        if not s:
+            return None
+        items = str(s).rsplit(':', 2)
+        if len(items) < 2:
+            return None  # although maybe we could guess the port too?
+        host = items[0]
+        port = items[1]
+        if len(items) >= 3:
+            protocol = items[2]
+        else:
+            protocol = PREFERRED_NETWORK_PROTOCOL
+        return ServerAddr(host=host, port=port, protocol=protocol)
+
+    def __str__(self):
+        return '{}:{}'.format(self.net_addr_str(), self.protocol)
+
+    def to_json(self) -> str:
+        return str(self)
+
+    def __repr__(self):
+        return f'<ServerAddr host={self.host} port={self.port} protocol={self.protocol}>'
+
+    def net_addr_str(self) -> str:
+        return self._net_addr_str
+
+    def __eq__(self, other):
+        if not isinstance(other, ServerAddr):
+            return False
+        return (self.host == other.host
+                and self.port == other.port
+                and self.protocol == other.protocol)
+
+    def __ne__(self, other):
+        return not (self == other)
+
+    def __hash__(self):
+        return hash((self.host, self.port, self.protocol))
 
 
-def serialize_server(host: str, port: Union[str, int], protocol: str) -> str:
-    return str(':'.join([host, str(port), protocol]))
+def _get_cert_path_for_host(*, config: 'SimpleConfig', host: str) -> str:
+    filename = host
+    try:
+        ip = ip_address(host)
+    except ValueError:
+        pass
+    else:
+        if isinstance(ip, IPv6Address):
+            filename = f"ipv6_{ip.packed.hex()}"
+    return os.path.join(config.path, 'certs', filename)
 
 
 class Interface(Logger):
 
     LOGGING_SHORTCUT = 'i'
 
-    def __init__(self, network: 'Network', server: str, proxy: Optional[dict]):
+    def __init__(self, *, network: 'Network', server: ServerAddr, proxy: Optional[dict]):
         self.ready = asyncio.Future()
         self.got_disconnected = asyncio.Future()
         self.server = server
-        self.host, self.port, self.protocol = deserialize_server(self.server)
-        self.port = int(self.port)
         Logger.__init__(self)
         assert network.config.path
-        self.cert_path = os.path.join(network.config.path, 'certs', self.host)
-        self.blockchain = None
-        self._requested_chunks = set()
+        self.cert_path = _get_cert_path_for_host(config=network.config, host=self.host)
+        self.blockchain = None  # type: Optional[Blockchain]
+        self._requested_chunks = set()  # type: Set[int]
         self.network = network
-        self._set_proxy(proxy)
-        self.session = None  # type: NotificationSession
+        self.proxy = MySocksProxy.from_proxy_dict(proxy)
+        self.session = None  # type: Optional[NotificationSession]
         self._ipaddr_bucket = None
 
+        # Latest block header and corresponding height, as claimed by the server.
+        # Note that these values are updated before they are verified.
+        # Especially during initial header sync, verification can take a long time.
+        # Failing verification will get the interface closed.
         self.tip_header = None
         self.tip = 0
 
+        self.fee_estimates_eta = {}
+
         # Dump network messages (only for this interface).  Set at runtime from the console.
         self.debug = False
 
         asyncio.run_coroutine_threadsafe(
-            self.network.main_taskgroup.spawn(self.run()), self.network.asyncio_loop)
-        self.group = SilentTaskGroup()
+            self.network.taskgroup.spawn(self.run()), self.network.asyncio_loop)
+        self.taskgroup = SilentTaskGroup()
+
+    @property
+    def host(self):
+        return self.server.host
+
+    @property
+    def port(self):
+        return self.server.port
+
+    @property
+    def protocol(self):
+        return self.server.protocol
 
     def diagnostic_name(self):
-        return f"{self.host}:{self.port}"
+        return self.server.net_addr_str()
 
-    def _set_proxy(self, proxy: dict):
-        if proxy:
-            username, pw = proxy.get('user'), proxy.get('password')
-            if not username or not pw:
-                auth = None
-            else:
-                auth = aiorpcx.socks.SOCKSUserAuth(username, pw)
-            addr = NetAddress(proxy['host'], proxy['port'])
-            if proxy['mode'] == "socks4":
-                self.proxy = aiorpcx.socks.SOCKSProxy(addr, aiorpcx.socks.SOCKS4a, auth)
-            elif proxy['mode'] == "socks5":
-                self.proxy = aiorpcx.socks.SOCKSProxy(addr, aiorpcx.socks.SOCKS5, auth)
-            else:
-                raise NotImplementedError  # http proxy not available with aiorpcx
-        else:
-            self.proxy = None
+    def __str__(self):
+        return f"<Interface {self.diagnostic_name()}>"
 
     async def is_server_ca_signed(self, ca_ssl_context):
         """Given a CA enforcing SSL context, returns True if the connection
@@ -270,11 +400,13 @@ class Interface(Logger):
     async def _try_saving_ssl_cert_for_first_time(self, ca_ssl_context):
         ca_signed = await self.is_server_ca_signed(ca_ssl_context)
         if ca_signed:
+            if self._get_expected_fingerprint():
+                raise InvalidOptionCombination("cannot use --serverfingerprint with CA signed servers")
             with open(self.cert_path, 'w') as f:
                 # empty file means this is CA signed, not self-signed
                 f.write('')
         else:
-            await self.save_certificate()
+            await self._save_certificate()
 
     def _is_saved_ssl_cert_available(self):
         if not os.path.exists(self.cert_path):
@@ -282,6 +414,8 @@ class Interface(Logger):
         with open(self.cert_path, 'r') as f:
             contents = f.read()
         if contents == '':  # CA signed
+            if self._get_expected_fingerprint():
+                raise InvalidOptionCombination("cannot use --serverfingerprint with CA signed servers")
             return True
         # pinned self-signed cert
         try:
@@ -296,11 +430,12 @@ class Interface(Logger):
             raise ErrorParsingSSLCert(e) from e
         try:
             x.check_date()
-            return True
         except x509.CertificateError as e:
             self.logger.info(f"certificate has expired: {e}")
             os.unlink(self.cert_path)  # delete pinned cert only in this case
             return False
+        self._verify_certificate_fingerprint(bytearray(b))
+        return True
 
     async def _get_ssl_context(self):
         if self.protocol != 's':
@@ -342,7 +477,7 @@ class Interface(Logger):
                 self.ready.cancel()
         return wrapper_func
 
-    @ignore_exceptions  # do not kill main_taskgroup
+    @ignore_exceptions  # do not kill network.taskgroup
     @log_exceptions
     @handle_disconnect
     async def run(self):
@@ -354,10 +489,16 @@ class Interface(Logger):
         try:
             await self.open_session(ssl_context)
         except (asyncio.CancelledError, ConnectError, aiorpcx.socks.SOCKSError) as e:
-            self.logger.info(f'disconnecting due to: {repr(e)}')
+            # make SSL errors for main interface more visible (to help servers ops debug cert pinning issues)
+            if (isinstance(e, ConnectError) and isinstance(e.__cause__, ssl.SSLError)
+                    and self.is_main_server() and not self.network.auto_connect):
+                self.logger.warning(f'Cannot connect to main server due to SSL error '
+                                    f'(maybe cert changed compared to "{self.cert_path}"). Exc: {repr(e)}')
+            else:
+                self.logger.info(f'disconnecting due to: {repr(e)}')
             return
 
-    def mark_ready(self):
+    def _mark_ready(self) -> None:
         if self.ready.cancelled():
             raise GracefulDisconnect('conn establishment was too slow; *ready* future was cancelled')
         if self.ready.done():
@@ -375,13 +516,14 @@ class Interface(Logger):
 
         self.ready.set_result(1)
 
-    async def save_certificate(self):
+    async def _save_certificate(self) -> None:
         if not os.path.exists(self.cert_path):
             # we may need to retry this a few times, in case the handshake hasn't completed
             for _ in range(10):
-                dercert = await self.get_certificate()
+                dercert = await self._fetch_certificate()
                 if dercert:
                     self.logger.info("succeeded in getting cert")
+                    self._verify_certificate_fingerprint(dercert)
                     with open(self.cert_path, 'w') as f:
                         cert = ssl.DER_cert_to_PEM_cert(dercert)
                         # workaround android bug
@@ -397,15 +539,29 @@ class Interface(Logger):
             else:
                 raise GracefulDisconnect("could not get certificate after 10 tries")
 
-    async def get_certificate(self):
+    async def _fetch_certificate(self) -> bytes:
         sslc = ssl.SSLContext()
-        try:
-            async with _RSClient(session_factory=RPCSession,
-                                 host=self.host, port=self.port,
-                                 ssl=sslc, proxy=self.proxy) as session:
-                return session.transport._asyncio_transport._ssl_protocol._sslpipe._sslobj.getpeercert(True)
-        except ValueError:
-            return None
+        async with _RSClient(session_factory=RPCSession,
+                             host=self.host, port=self.port,
+                             ssl=sslc, proxy=self.proxy) as session:
+            asyncio_transport = session.transport._asyncio_transport  # type: asyncio.BaseTransport
+            ssl_object = asyncio_transport.get_extra_info("ssl_object")  # type: ssl.SSLObject
+            return ssl_object.getpeercert(binary_form=True)
+
+    def _get_expected_fingerprint(self) -> Optional[str]:
+        if self.is_main_server():
+            return self.network.config.get("serverfingerprint")
+
+    def _verify_certificate_fingerprint(self, certificate):
+        expected_fingerprint = self._get_expected_fingerprint()
+        if not expected_fingerprint:
+            return
+        fingerprint = hashlib.sha256(certificate).hexdigest()
+        fingerprints_match = fingerprint.lower() == expected_fingerprint.lower()
+        if not fingerprints_match:
+            util.trigger_callback('cert_mismatch')
+            raise ErrorSSLCertFingerprintMismatch('Refusing to connect to server due to cert fingerprint mismatch')
+        self.logger.info("cert fingerprint verification passed")
 
     async def get_block_header(self, height, assert_mode):
         self.logger.info(f'requesting block header {height} in mode {assert_mode}')
@@ -414,7 +570,9 @@ class Interface(Logger):
         res = await self.session.send_request('blockchain.block.header', [height], timeout=timeout)
         return blockchain.deserialize_header(bytes.fromhex(res), height)
 
-    async def request_chunk(self, height, tip=None, *, can_return_early=False):
+    async def request_chunk(self, height: int, tip=None, *, can_return_early=False):
+        if not is_non_negative_integer(height):
+            raise Exception(f"{repr(height)} is not a block height")
         index = height // 2016
         if can_return_early and index in self._requested_chunks:
             return
@@ -427,8 +585,17 @@ class Interface(Logger):
             self._requested_chunks.add(index)
             res = await self.session.send_request('blockchain.block.headers', [index * 2016, size])
         finally:
-            try: self._requested_chunks.remove(index)
-            except KeyError: pass
+            self._requested_chunks.discard(index)
+        assert_dict_contains_field(res, field_name='count')
+        assert_dict_contains_field(res, field_name='hex')
+        assert_dict_contains_field(res, field_name='max')
+        assert_non_negative_integer(res['count'])
+        assert_non_negative_integer(res['max'])
+        assert_hex_str(res['hex'])
+        if len(res['hex']) != HEADER_SIZE * 2 * res['count']:
+            raise RequestCorrupted('inconsistent chunk hex and count')
+        if res['count'] != size:
+            raise RequestCorrupted(f"expected {size} headers but only got {res['count']}")
         conn = self.blockchain.connect_chunk(index, res['hex'])
         if not conn:
             return conn, 0
@@ -456,8 +623,9 @@ class Interface(Logger):
             self.logger.info(f"connection established. version: {ver}")
 
             try:
-                async with self.group as group:
+                async with self.taskgroup as group:
                     await group.spawn(self.ping)
+                    await group.spawn(self.request_fee_estimates)
                     await group.spawn(self.run_fetch_blocks)
                     await group.spawn(self.monitor_connection)
             except aiorpcx.jsonrpc.RPCError as e:
@@ -478,6 +646,21 @@ class Interface(Logger):
             await asyncio.sleep(300)
             await self.session.send_request('server.ping')
 
+    async def request_fee_estimates(self):
+        from .simple_config import FEE_ETA_TARGETS
+        from .bitcoin import COIN
+        while True:
+            async with TaskGroup() as group:
+                fee_tasks = []
+                for i in FEE_ETA_TARGETS:
+                    fee_tasks.append((i, await group.spawn(self.session.send_request('blockchain.estimatefee', [i]))))
+            for nblock_target, task in fee_tasks:
+                fee = int(task.result() * COIN)
+                if fee < 0: continue
+                self.fee_estimates_eta[nblock_target] = fee
+            self.network.update_fee_estimates()
+            await asyncio.sleep(60)
+
     async def close(self):
         if self.session:
             await self.session.close()
@@ -495,9 +678,11 @@ class Interface(Logger):
             self.tip = height
             if self.tip < constants.net.max_checkpoint():
                 raise GracefulDisconnect('server tip below max checkpoint')
-            self.mark_ready()
+            self._mark_ready()
             await self._process_header_at_tip()
-            self.network.trigger_callback('network_updated')
+            # header processing done
+            util.trigger_callback('blockchain_updated')
+            util.trigger_callback('network_updated')
             await self.network.switch_unwanted_fork_interface()
             await self.network.switch_lagging_interface()
 
@@ -512,7 +697,6 @@ class Interface(Logger):
             # in the simple case, height == self.tip+1
             if height <= self.tip:
                 await self.sync_until(height)
-        self.network.trigger_callback('blockchain_updated')
 
     async def sync_until(self, height, next_height=None):
         if next_height is None:
@@ -527,7 +711,7 @@ class Interface(Logger):
                         raise GracefulDisconnect('server chain conflicts with checkpoints or genesis')
                     last, height = await self.step(height)
                     continue
-                self.network.trigger_callback('network_updated')
+                util.trigger_callback('network_updated')
                 height = (height // 2016 * 2016) + num_headers
                 assert height <= next_height+1, (height, self.tip)
                 last = 'catchup'
@@ -670,11 +854,13 @@ class Interface(Logger):
             if self.is_tor():
                 return BUCKET_NAME_OF_ONION_SERVERS
             try:
-                ip_addr = ip_address(self.ip_addr())
+                ip_addr = ip_address(self.ip_addr())  # type: Union[IPv4Address, IPv6Address]
             except ValueError:
                 return ''
             if not ip_addr:
                 return ''
+            if ip_addr.is_loopback:  # localhost is exempt
+                return ''
             if ip_addr.version == 4:
                 slash16 = IPv4Network(ip_addr).supernet(prefixlen_diff=32-16)
                 return str(slash16)
@@ -687,6 +873,108 @@ class Interface(Logger):
             self._ipaddr_bucket = do_bucket()
         return self._ipaddr_bucket
 
+    async def get_merkle_for_transaction(self, tx_hash: str, tx_height: int) -> dict:
+        if not is_hash256_str(tx_hash):
+            raise Exception(f"{repr(tx_hash)} is not a txid")
+        if not is_non_negative_integer(tx_height):
+            raise Exception(f"{repr(tx_height)} is not a block height")
+        # do request
+        res = await self.session.send_request('blockchain.transaction.get_merkle', [tx_hash, tx_height])
+        # check response
+        block_height = assert_dict_contains_field(res, field_name='block_height')
+        merkle = assert_dict_contains_field(res, field_name='merkle')
+        pos = assert_dict_contains_field(res, field_name='pos')
+        # note: tx_height was just a hint to the server, don't enforce the response to match it
+        assert_non_negative_integer(block_height)
+        assert_non_negative_integer(pos)
+        assert_list_or_tuple(merkle)
+        for item in merkle:
+            assert_hash256_str(item)
+        return res
+
+    async def get_transaction(self, tx_hash: str, *, timeout=None) -> str:
+        if not is_hash256_str(tx_hash):
+            raise Exception(f"{repr(tx_hash)} is not a txid")
+        raw = await self.session.send_request('blockchain.transaction.get', [tx_hash], timeout=timeout)
+        # validate response
+        tx = Transaction(raw)
+        try:
+            tx.deserialize()  # see if raises
+        except Exception as e:
+            raise RequestCorrupted(f"cannot deserialize received transaction (txid {tx_hash})") from e
+        if tx.txid() != tx_hash:
+            raise RequestCorrupted(f"received tx does not match expected txid {tx_hash} (got {tx.txid()})")
+        return raw
+
+    async def get_history_for_scripthash(self, sh: str) -> List[dict]:
+        if not is_hash256_str(sh):
+            raise Exception(f"{repr(sh)} is not a scripthash")
+        # do request
+        res = await self.session.send_request('blockchain.scripthash.get_history', [sh])
+        # check response
+        assert_list_or_tuple(res)
+        for tx_item in res:
+            assert_dict_contains_field(tx_item, field_name='height')
+            assert_dict_contains_field(tx_item, field_name='tx_hash')
+            assert_integer(tx_item['height'])
+            assert_hash256_str(tx_item['tx_hash'])
+            if tx_item['height'] in (-1, 0):
+                assert_dict_contains_field(tx_item, field_name='fee')
+                assert_non_negative_integer(tx_item['fee'])
+        return res
+
+    async def listunspent_for_scripthash(self, sh: str) -> List[dict]:
+        if not is_hash256_str(sh):
+            raise Exception(f"{repr(sh)} is not a scripthash")
+        # do request
+        res = await self.session.send_request('blockchain.scripthash.listunspent', [sh])
+        # check response
+        assert_list_or_tuple(res)
+        for utxo_item in res:
+            assert_dict_contains_field(utxo_item, field_name='tx_pos')
+            assert_dict_contains_field(utxo_item, field_name='value')
+            assert_dict_contains_field(utxo_item, field_name='tx_hash')
+            assert_dict_contains_field(utxo_item, field_name='height')
+            assert_non_negative_integer(utxo_item['tx_pos'])
+            assert_non_negative_integer(utxo_item['value'])
+            assert_non_negative_integer(utxo_item['height'])
+            assert_hash256_str(utxo_item['tx_hash'])
+        return res
+
+    async def get_balance_for_scripthash(self, sh: str) -> dict:
+        if not is_hash256_str(sh):
+            raise Exception(f"{repr(sh)} is not a scripthash")
+        # do request
+        res = await self.session.send_request('blockchain.scripthash.get_balance', [sh])
+        # check response
+        assert_dict_contains_field(res, field_name='confirmed')
+        assert_dict_contains_field(res, field_name='unconfirmed')
+        assert_non_negative_integer(res['confirmed'])
+        assert_non_negative_integer(res['unconfirmed'])
+        return res
+
+    async def get_txid_from_txpos(self, tx_height: int, tx_pos: int, merkle: bool):
+        if not is_non_negative_integer(tx_height):
+            raise Exception(f"{repr(tx_height)} is not a block height")
+        if not is_non_negative_integer(tx_pos):
+            raise Exception(f"{repr(tx_pos)} should be non-negative integer")
+        # do request
+        res = await self.session.send_request(
+            'blockchain.transaction.id_from_pos',
+            [tx_height, tx_pos, merkle],
+        )
+        # check response
+        if merkle:
+            assert_dict_contains_field(res, field_name='tx_hash')
+            assert_dict_contains_field(res, field_name='merkle')
+            assert_hash256_str(res['tx_hash'])
+            assert_list_or_tuple(res['merkle'])
+            for node_hash in res['merkle']:
+                assert_hash256_str(node_hash)
+        else:
+            assert_hash256_str(res)
+        return res
+
 
 def _assert_header_does_not_check_against_any_chain(header: dict) -> None:
     chain_bad = blockchain.check_header(header) if 'mock' not in header else header['mock']['check'](header)
